//Sketch per la trasmissione di dati di temperatura al sito ThingSpeak
//allegato al libro RETI DOMESTICHE di Pier Calderan (Feltrinelli 2015)
//Caricare lo sketch in Arduino UNO +  Ethernet Shield.
//Collegare il sensore di temperatura tipo LM35 alla porta A0, +5V e GND di Arduino UNO

#include <SPI.h>
#include <Ethernet.h>
#include <Wire.h>
byte mac[] = { 0xD4, 0x28, 0xB2, 0xFF, 0xA0, 0xA1 }; // Indirizzo mac dello shield Ethernet
char thingSpeakAddress[] = "api.thingspeak.com"; // DNS di ThingSpeak

String writeAPIKey = "CHIAVE_API"; //Scrivere qui la chiave API di ThingSpeak

const int updateThingSpeakInterval = 30000; //intervallo in millisecondi per il caricamento al sito ThingSpeak
long lastConnectionTime = 0; 
boolean lastConnected = false;
int failedCounter = 0;

EthernetClient client; //istanza del client Ethernet

void setup(){
Serial.begin(9600);// inizializzazione del monitor seriale
startEthernet(); //inizializzazione Ethernet shield 
}
void loop(){

//*************Lettura della temperatura
int valore = analogRead(A0) * 5; //lettura del valore della porta analogica * 5 volt di riferimento
int t = valore /= 10; //il sensore fornisce 1 grado ogni 10 millivolt, per cui il valore letto viene diviso per 10
int tF = (t * 9.0 / 5.0) + 32.0; // conversione in gradi Fahrenheit
//*************

if (client.available())
{ char c = client.read(); }
if (!client.connected() && lastConnected) // disconnessine da ThingSpeak
{ Serial.println("... disconnesso"); Serial.println();//stampa a capo
client.stop(); //termine della connessione
}
if(!client.connected() && (millis() - lastConnectionTime > updateThingSpeakInterval)) // upload al sito
{

  updateThingSpeak("1="+String(t)+"&2="+String(tF));//stringa aggiornata per il caricamento

Serial.println("Temp C: "+String(t)); //stampa gradi Celsius sul monitor seriale
Serial.println("Temp F: "+String(tF)); //stampa gradi Fahrenheit sul monitor seriale
Serial.println();// stampa a capo
}
if (failedCounter > 3 ) {startEthernet();}
lastConnected = client.connected();
}

//Funzione per la scrittura dei dati HTTP
void updateThingSpeak(String tsData){
if (client.connect(thingSpeakAddress, 80)){
client.print("POST /update HTTP/1.1\n"); //scrittura dati HTTP
client.print("Host: api.thingspeak.com\n");
client.print("Connection: close\n");
client.print("X-THINGSPEAKAPIKEY: "+writeAPIKey+"\n");
client.print("Content-Type: application/x-www-form-urlencoded\n");
client.print("Content-Length: ");
client.print(tsData.length());
client.print("\n\n");
client.print(tsData);

lastConnectionTime = millis();

if (client.connected())
{
Serial.println("Collegamento a ThingSpeak..."); Serial.println();
failedCounter = 0;
}
else
{
failedCounter++;
Serial.println("Connessione fallita ("+String(failedCounter, DEC)+")"); Serial.println();
}

} 
else 
{
failedCounter++;
Serial.println("Connessione fallita ("+String(failedCounter, DEC)+")"); 
Serial.println();
lastConnectionTime = millis();
}
}

//Inizializzazione della connessione
void startEthernet()
{
client.stop();
Serial.println("Collegamento di Arduino alla rete..."); 
Serial.println();
delay(1000); //ritardo di un secondo
if (Ethernet.begin(mac) == 0) // collegamento per ottenere un IP da DHCP
{
Serial.println("DHCP fallito, riavvia Arduino o riprova"); 
Serial.println();
} 
else 
{
Serial.println("Arduino connesso alla rete in DHCP"); 
Serial.println();
} 
delay(1000); //ritardo di un secondo
}


